@@ -1,6 +1,10 @@
 # -*- coding: utf-8 -*-
+import os
 import protocol
 import xml.sax.saxutils
+import gettext
+
+_ = gettext.gettext
 
 class Channel:
     """
@@ -43,10 +47,9 @@ class ClientFeedback:
     def onConnected(self):
         pass
 
-    def onDisconnected(self):
+    def onDisconnected(self, reason):
         pass
     
-    
     def openChannel(self, feedback):
         """Open a channel to the GGZ server.
         
@@ -54,19 +57,18 @@ class ClientFeedback:
         """
         pass
     
-    def getUsername(self):
-        """Called when the username is required.
+    def getLogin(self):
+        """Called when the login credentials are required.
         
-        Returns the username to log in with.
+        Returns the username and password to log in with.
+        If the password is None then log in as guest.
         """
-        return 'test'
+        return ('test', None)
     
     def getPassword(self, username):
         """Called when a password is required.
         
         'username' is the username the password is required for.
-        
-        Returns the password for this user or None to abort login.
         """
         return None
     
@@ -112,63 +114,56 @@ class ClientFeedback:
         pass
 
 class Game:
-    id = ''
     
-    name = ''
-    version = ''
-    
-    protocol_engine = ''
-    protocol_version = ''
-    
-    nPlayers = 0
-    
-    author = ''
-    url = ''
+    def __init__(self):
+        self.id = ''
+        self.name = ''
+        self.version = ''
+        self.protocol_engine = ''
+        self.protocol_version = ''
+        self.nPlayers = 0
+        self.author = ''
+        self.url = ''
 
 class Room:
-    id = ''
     
-    game = None
-    
-    nPlayers = 0
+    def __init__(self):
+        self.id = ''
+        self.game = None    
+        self.nPlayers = 0
 
 class Table:
-    id = ''
-    
-    room = ''
-    
-    game = None
-    
-    status = ''
-    
-    description = ''
     
     def __init__(self, nSeats):
+        self.id = ''
+        self.room = ''
+        self.game = None
+        self.status = ''
+        self.description = ''
         self.seats = []
         for i in xrange(nSeats):
             seat = Seat()
             self.seats.append(seat)
 
 class Seat:
-    type = ''
     
-    user = ''
+    def __init__(self):
+        self.type = ''
+        self.user = ''
 
 class Player:
-    name = ''
     
-    type = ''
-    
-    table = None
-    
-    perms = 0
-    
-    lag = 0
-    
-    room = None
-    lastRoom = None
+    def __init__(self):
+        self.name = ''
+        self.type = ''
+        self.table = None
+        self.perms = 0
+        self.lag = 0
+        self.room = None
+        self.lastRoom = None
     
 class MainChannel(ChannelFeedback, protocol.ParserFeedback):
+
     def __init__(self, client):
         self.client = client
         self.decoder = protocol.Decoder(self)
@@ -185,15 +180,20 @@ class MainChannel(ChannelFeedback, protocol.ParserFeedback):
         assert(self.client.state is self.client.STATE_DISCONNECTED)
         self.client.feedback.onConnected()
         self.client.setState(self.client.STATE_START_SESSION)
+        
+        try:
+            language = os.environ['LANG']
+        except KeyError:
+            language = 'C'
+
         self.send("<?xml version='1.0' encoding='UTF-8'?>\n")
         self.send("<SESSION>\n")
-        self.send("<LANGUAGE>en_NZ.UTF-8</LANGUAGE>\n")
-        self.client.username = self.client.feedback.getUsername()
-        password = self.client.feedback.getPassword(self.client.username)
-        if password:
-            self.client._login(self.client.username, password)
-        else:
+        self.send("<LANGUAGE>%s</LANGUAGE>\n" % xml.sax.saxutils.escape(language))
+        (self.client.username, self.client.password) = self.client.feedback.getLogin()
+        if self.client.password is None:
             self.client._loginGuest(self.client.username)
+        else:
+            self.client._login(self.client.username, self.client.password)
         
     def onResult(self, action, code):
         if action == 'login':
@@ -201,20 +201,21 @@ class MainChannel(ChannelFeedback, protocol.ParserFeedback):
                 self.client.setState(self.client.STATE_LIST_GAMES)
             else:
                 if code == 'usr lookup':
-                    self.client.setState(self.client.STATE_START_SESSION)
-                    password = self.client.feedback.getPassword(self.client.username)
-                    if password is not None:
-                        self.client._login(self.client.username, password)
-                        return
-                    print 'Failed to login: Require password'
+                    # Translators: GGZ disconnection error when the supplied password is incorrect
+                    self.client.close(_('Incorrect password'))
+                    #FIXME: Prompt for a password
+                    #self.client.setState(self.client.STATE_GET_PASSWORD)
 
                 elif code == 'already logged in':
-                    print 'Failed to login: Already logged in'
-
-                else:
-                    print 'Failed to login: %s' % code
+                    # Translators: GGZ disconnection error when the selected account is already in use
+                    self.client.close(_('Account in use'))
+                    #FIXME: If guest then prompt for a new user or mangle username until one can be found
                     
-                self.client.setState(self.client.STATE_DISCONNECTED)
+                elif code == 'wrong login type':
+                    self.client.setState(self.client.STATE_GET_PASSWORD)
+                
+                else:
+                    self.client.close(code)
 
         elif action == 'enter':
             if code != 'ok':
@@ -287,8 +288,12 @@ class MainChannel(ChannelFeedback, protocol.ParserFeedback):
         self.client.feedback.onChat(chatType, sender, text)
         
     def onJoin(self, tableId, isSpectator):
+        try:
+            table = self._getTable(tableId)
+        except KeyError:
+            print "Unknown JOIN with TABLE='%s'" % tableId
+            return
         self.client.setState(self.client.STATE_AT_TABLE)
-        table = self.client.tables[tableId]
         g = self.client.feedback.onJoin(table, isSpectator, self.client.channel)
         self.client.channel.setGame(g)
         
@@ -308,12 +313,14 @@ class MainChannel(ChannelFeedback, protocol.ParserFeedback):
         self.client.games[gameId] = game
 
     def roomAdded(self, roomId, gameId, name, description, nPlayers):
+        try:
+            game = self._getGame(gameId, optional = True)
+        except KeyError:
+            print "Unknown ROOM ADD with GAME='%s'" % gameId
+            return
         room = Room()
         room.id = roomId
-        try:
-            room.game = self.client.games[gameId]
-        except KeyError:
-            room.game = None
+        room.game = game
         room.name = name
         room.description = description
         room.nPlayers = int(nPlayers)
@@ -321,27 +328,45 @@ class MainChannel(ChannelFeedback, protocol.ParserFeedback):
         self.client.feedback.roomAdded(room)
 
     def roomPlayersUpdate(self, roomId, nPlayers):
-        room = self.client.rooms[roomId]
+        try:
+            room = self._getRoom(roomId)
+        except KeyError:
+            print "Unknown ROOM PLAYER UPDATE with ROOM='%s'" % roomId
+            return
         room.nPlayers = int(nPlayers)
         self.client.feedback.roomUpdated(room)
 
-    def tableAdded(self, roomId, tableId, gameId, status, nSeats, description):
+    def tableAdded(self, roomId, tableId, gameId, status, nSeats, description): 
+        try:
+            room = self._getRoom(roomId)
+            game = self._getGame(gameId)
+        except KeyError:
+            print "Unknown TABLE ADD with ROOM='%s' GAME='%s'" % (roomId, gameId)
+            return
         table = Table(int(nSeats))
         table.id = tableId
-        table.room = self.client.rooms[roomId]
-        table.game = self.client.games[gameId]
+        table.room = room
+        table.game = game
         table.status = status
         table.description = description
         self.client.tables[tableId] = table
         self.client.feedback.tableAdded(table)
 
     def tableStatusChanged(self, tableId, status):
-        table = self.client.tables[tableId]
+        try:
+            table = self._getTable(tableId)
+        except KeyError:
+            print "Unknown TABLE STATUS with TABLE='%s'" % tableId
+            return
         table.status = status
         self.client.feedback.tableUpdated(table)
 
     def seatChanged(self, roomId, tableId, seatId, seatType, user):
-        table = self.client.tables[tableId]
+        try:
+            table = self._getTable(tableId)
+        except KeyError:
+            print "Unknown SEAT CHANGE with TABLE='%s'" % tableId
+            return
         if table.room.id != roomId:
             return
         seat = table.seats[int(seatId)]
@@ -352,81 +377,100 @@ class MainChannel(ChannelFeedback, protocol.ParserFeedback):
     def tableRemoved(self, tableId):
         try:
             table = self.client.tables.pop(tableId)
-        except KeyError:
+        except KeyError: 
+            print "Unknown TABLE REMOVE with TABLE='%s'" % tableId           
             # We do not know of this table - this could occur if we receive a
             # table remove event before we get the table list.
             return
         self.client.feedback.tableRemoved(table)
         
-    def onPlayerList(self, room, players):
+    def onPlayerList(self, roomId, players):
+        try:
+            room = self._getRoom(roomId)
+            for p in players:
+                _ = self._getTable(p.tableId, optional = True)
+        except KeyError: 
+            print "Unknown PLAYER LIST with ROOM='%s'" % roomId
+            return
         self.client.players = {}
-        r = self.client.rooms[room]
         for p in players:
             player = Player()
             player.name = p.name
             player.type = p.type
-            try:
-                player.table = self.client.tables[p.table]
-            except KeyError:
-                player.table = None
+            player.table = self._getTable(p.tableId, optional = True)
             player.perms = p.perms
             player.lag = p.lag
-            player.room = r
+            player.room = room
             self.client.players[player.name] = player
 
-        r.nPlayers = len(players)
-        self.client.feedback.roomUpdated(r)
+        room.nPlayers = len(players)
+        self.client.feedback.roomUpdated(room)
 
-    def playerAdded(self, name, playerType, tableId, perms, lag, room, fromRoom):
+    def playerAdded(self, name, playerType, tableId, perms, lag, roomId, fromRoomId):
+        try:
+            room = self._getRoom(roomId, optional = True)
+            lastRoom = self.client.rooms[fromRoomId]
+            table = self._getTable(tableId, optional = True)
+        except KeyError:
+            print "Unknown PLAYER ADD with ROOM='%s' LASTROOM='%s' TABLE='%s'" % (roomId, fromRoomId, tableId)
+            return
         player = Player()
         player.name = name
         player.type = playerType
-        try:
-            player.table = self.client.tables[tableId]
-        except KeyError:
-            player.table = None
+        player.table = table
         player.perms = perms
         player.lag = lag
-        player.room = self.client.rooms[room]
-        player.room.nPlayers += 1
+        player.room = room
+        player.lastRoom = lastRoom
         self.client.players[player.name] = player
 
-        try:
-            player.lastRoom = self.client.rooms[fromRoom]
-        except KeyError:
-            player.lastRoom = None
-        else:
+        if player.lastRoom is not None:
             player.lastRoom.nPlayers -= 1
             self.client.feedback.roomUpdated(player.lastRoom)
-
-        self.client.feedback.roomUpdated(player.room)
+        if player.room is not None:
+            player.room.nPlayers += 1
+            self.client.feedback.roomUpdated(player.room)
         self.client.feedback.playerAdded(player)
 
-    def playerRemoved(self, name, room, toRoom):
+    def playerRemoved(self, name, roomId, toRoomId):
         try:
             player = self.client.players.pop(name)
+            room = self._getRoom(toRoomId, optional = True)
         except KeyError:
+            print "Unknown PLAYER REMOVE with NAME='%s' ROOM='%s' TOROOM='%s'" % (name, roomId, toRoomId)
             # We do not know of this player - this could occur if we receive a
             # player remove event before we get the player list.
             return
             
-        player.room.nPlayers -= 1
         player.lastRoom = player.room
-        try:
-            player.room = self.client.rooms[toRoom]
-        except KeyError:
-            player.room = None
-        else:
+        player.room = room
+        if player.room is not None:
             player.room.nPlayers += 1
             self.client.feedback.roomUpdated(player.room)
-        
-        self.client.feedback.roomUpdated(player.lastRoom)
+        if player.lastRoom is not None:
+            player.lastRoom.nPlayers -= 1
+            self.client.feedback.roomUpdated(player.lastRoom)
         self.client.feedback.playerRemoved(player)
 
     def closed(self, errno = 0):
-        print 'SEVERE: GGZ connection closed: error %d' % errno
-        self.client.setState(self.client.STATE_DISCONNECTED)
+        # Translators: GGZ disconnection error when the network link has broken. %s is the system provided error
+        self.client.close(_('Connection closed: %s') % os.strerror(errno))
+       
+    def _getGame(self, gameId, optional = False):
+        if optional and gameId == '-1':
+            return None
+        return self.client.games[gameId]
+
+    def _getRoom(self, roomId, optional = False):
+        if optional and roomId == '-1':
+            return None
+        return self.client.rooms[roomId]    
         
+    def _getTable(self, tableId, optional = False):
+        if optional and tableId == '-1':
+            return None
+        return self.client.tables[tableId]
+    
 class GameChannel(ChannelFeedback, protocol.ParserFeedback):
     
     def __init__(self, client, command):
@@ -459,8 +503,8 @@ class GameChannel(ChannelFeedback, protocol.ParserFeedback):
         else:
             self.game.registerIncomingData(data)
 
-    def onDisconnected(self):
-        print '!'
+    def onDisconnected(self, reason):
+        print 'Disconnected: %s' % reason
 
     def send(self, data, isBinary = False):
         self.controller.send(data, isBinary)
@@ -483,6 +527,7 @@ class Client:
     STATE_DISCONNECTED        = 'DISCONNECTED'
     STATE_START_SESSION       = 'START_SESSION'
     STATE_LOGIN               = 'LOGIN'
+    STATE_GET_PASSWORD        = 'GET_PASSWORD'    
     STATE_LIST_GAMES          = 'LIST_GAMES'
     STATE_LIST_ROOMS          = 'LIST_ROOMS'
     STATE_READY               = 'READY'
@@ -514,7 +559,8 @@ class Client:
         """
         return self.state is self.STATE_READY
 
-    def close(self):
+    def close(self, error):
+        self.disconnectionError = error
         self.setState(self.STATE_DISCONNECTED)
         
     def isBusy(self):
@@ -534,8 +580,10 @@ class Client:
             self.mainChannel.send("<LIST TYPE='table'/>\n")
         elif state is self.STATE_LIST_PLAYERS:
             self.mainChannel.send("<LIST TYPE='player'/>\n")
+        elif state is self.STATE_GET_PASSWORD:
+            self.feedback.getPassword(self.username)
         elif state is self.STATE_DISCONNECTED:
-            self.feedback.onDisconnected()
+            self.feedback.onDisconnected(self.disconnectionError)
             self.mainChannel.controller.close()
 
     def start(self):
@@ -543,8 +591,16 @@ class Client:
         self.mainChannel = MainChannel(self)
         self.mainChannel.controller = self.feedback.openChannel(self.mainChannel)
         
+    def setPassword(self, password):
+        assert(self.state is self.STATE_GET_PASSWORD)
+        if password is None:
+            # Translators: GGZ disconnection error when a password was required for the selected account
+            self.close(_('A password is required'))
+        else:
+            self._login(self.username, password)
+        
     def _login(self, username, password):
-        assert(self.state is self.STATE_START_SESSION)
+        assert(self.state is self.STATE_START_SESSION or self.state is self.STATE_GET_PASSWORD)
         self.setState(self.STATE_LOGIN)
         username = xml.sax.saxutils.escape(username)
         password = xml.sax.saxutils.escape(password)
