@@ -153,7 +153,7 @@ save_state_cb (EggSMClient *client,
   int i;
   int xpos, ypos;
 
-  gdk_window_get_origin (app->window, &xpos, &ypos);
+  gdk_window_get_origin (gtk_widget_get_window (app), &xpos, &ypos);
 
   i = 0;
   argv[i++] = (char *) client_data;
@@ -200,6 +200,7 @@ main (int argc, char *argv[])
   GOptionContext *context;
   struct timeval tv;
   gint i;
+  gchar *config;
   gboolean retval;
   GError *error = NULL;
 #ifdef WITH_SMCLIENT
@@ -212,7 +213,9 @@ main (int argc, char *argv[])
   gettimeofday (&tv, NULL);
   srand (tv.tv_usec);
 
+#ifdef ENABLE_SETGID
   setgid_io_init ();
+#endif
 
   context = g_option_context_new (NULL);
 #if GLIB_CHECK_VERSION (2, 12, 0)
@@ -314,15 +317,12 @@ main (int argc, char *argv[])
 
   if (!load_game_configs ()) {
     /* Oops, no configs, we probably haven't been installed properly. */
-    errordialog = gtk_message_dialog_new (NULL, 0, GTK_MESSAGE_ERROR,
+    errordialog = gtk_message_dialog_new_with_markup (NULL, 0, GTK_MESSAGE_ERROR,
 					  GTK_BUTTONS_OK,
 					  "<b>%s</b>\n\n%s",
 					  _("No game data could be found."),
 					  _
 					  ("The program Robots was unable to find any valid game configuration files. Please check that the program is installed correctly."));
-    gtk_label_set_use_markup (GTK_LABEL
-			      (GTK_MESSAGE_DIALOG (errordialog)->label),
-			      TRUE);
     gtk_window_set_resizable (GTK_WINDOW (errordialog), FALSE);
     gtk_dialog_run (GTK_DIALOG (errordialog));
     exit (1);
@@ -332,7 +332,7 @@ main (int argc, char *argv[])
 
   if (!load_game_graphics ()) {
     /* Oops, no graphics, we probably haven't been installed properly. */
-    errordialog = gtk_message_dialog_new (GTK_WINDOW (app),
+    errordialog = gtk_message_dialog_new_with_markup (GTK_WINDOW (app),
 					  GTK_DIALOG_MODAL,
 					  GTK_MESSAGE_ERROR,
 					  GTK_BUTTONS_OK,
@@ -341,10 +341,6 @@ main (int argc, char *argv[])
 					  ("Some graphics files are missing or corrupt."),
 					  _
 					  ("The program Robots was unable to load all the necessary graphics files. Please check that the program is installed correctly."));
-    gtk_label_set_use_markup (GTK_LABEL
-			      (GTK_MESSAGE_DIALOG (errordialog)->label),
-			      TRUE);
-
     gtk_dialog_run (GTK_DIALOG (errordialog));
     exit (1);
   }
@@ -361,10 +357,13 @@ main (int argc, char *argv[])
 
   if (cmdline_config) {
     for (i = 0; i < num_game_configs (); ++i) {
-      if (!strcmp (cmdline_config, game_config_name (i))) {
+      config = game_config_name (i);
+      if (!strcmp (cmdline_config, config)) {
 	properties_set_config (i);
+	g_free (config);
 	break;
       }
+      g_free (config);
     }
   }
 
