@@ -1311,7 +1311,7 @@ Tetris::gameAbout(GtkAction *action, void *d)
 			       "authors", authors,
 			       "documenters", documenters,
 			       "translator-credits", _("translator-credits"),
-			       "logo-icon-name", "gnome-gnometris",
+			       "logo-icon-name", "gnome-quadrapassel",
 			       "website", "http://www.gnome.org/projects/gnome-games/",
 			       "wrap-license", TRUE,
 			       NULL);
